Package: epiNEM
Type: Package
Title: epiNEM
Version: 1.36.0
Authors@R: c(person("Martin", "Pirkl",
    email = "martinpirkl@yahoo.de",
    role = c("aut", "cre")),
    person("Madeline", "Diekmann",
    role = c("aut")))
Author: Madeline Diekmann & Martin Pirkl
Maintainer: Martin Pirkl <martinpirkl@yahoo.de>
Description: epiNEM is an extension of the original Nested Effects
    Models (NEM). EpiNEM is able to take into account double
    knockouts and infer more complex network signalling pathways. 
    It is tailored towards large scale double knock-out screens.
Depends: R (>= 4.1)
License: GPL-3
Encoding: UTF-8
LazyData: true
biocViews: Pathways, SystemsBiology, NetworkInference, Network
RoxygenNote: 7.3.1
Imports: BoutrosLab.plotting.general, BoolNet, e1071, gtools, stats,
        igraph, utils, lattice, latticeExtra, RColorBrewer, pcalg,
        minet, grDevices, graph, mnem, latex2exp
VignetteBuilder: knitr
Suggests: knitr, RUnit, BiocGenerics, STRINGdb, devtools, rmarkdown,
        GOSemSim, AnnotationHub, org.Sc.sgd.db, BiocStyle
BugReports: https://github.com/cbg-ethz/epiNEM/issues
URL: https://github.com/cbg-ethz/epiNEM/
git_url: https://git.bioconductor.org/packages/epiNEM
git_branch: RELEASE_3_23
git_last_commit: 73fc1ee
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-06-02
NeedsCompilation: no
Packaged: 2026-06-02 18:24:23 UTC; biocbuild
Built: R 4.6.0; ; 2026-06-03 13:36:04 UTC; unix
