# Generated by roxygen2: do not edit by hand

S3method(plot,epiNEM)
S3method(plot,epiScreen)
S3method(plot,epiSim)
export(AddLogicGates)
export(CreateExtendedAdjacency)
export(CreateRandomGraph)
export(CreateTopology)
export(ExtendTopology)
export(GenerateData)
export(HeatmapOP)
export(Mll)
export(SimEpiNEM)
export(epiAnno)
export(epiNEM)
export(epiScreen)
export(perm.rank.test)
export(rank.enrichment)
import(RColorBrewer)
import(e1071)
import(grDevices)
import(lattice)
import(latticeExtra)
import(minet)
import(pcalg)
import(stats)
import(utils)
importFrom(BoolNet,getPathToAttractor)
importFrom(BoolNet,loadNetwork)
importFrom(BoutrosLab.plotting.general,legend.grob)
importFrom(graph,adj)
importFrom(gtools,combinations)
importFrom(igraph,graph.adjacency)
importFrom(igraph,layout.fruchterman.reingold)
importFrom(igraph,plot.igraph)
importFrom(latex2exp,TeX)
importFrom(mnem,mnem)
importFrom(mnem,transitive.reduction)
