% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/finalFunctions.R
\name{CreateTopology}
\alias{CreateTopology}
\title{Create Topology.}
\usage{
CreateTopology(single, double, force = TRUE)
}
\arguments{
\item{single}{number of single knockouts}

\item{double}{number of double knockouts}

\item{force}{if true the random model will have a sophisticated logical gate}
}
\value{
adjacency matrix
}
\description{
Create topology for a randomly generated pathway topology
}
\examples{
model <- CreateTopology(3, 1)
}
